;********************************************************
;*   AUTOREL/ASM : Automatic filter/driver relocator	*
;*							*
;* A collection of macro's and a front end for use by	*
;* programmers who wish to spend less time building 	*
;* address tables and the like.				*
;*							*
;*	Copyright 1982 by Jim Frimmel			*
;* released for non-commercial distribution only	*
;********************************************************
;
;	HOW TO USE THESE MACRO'S
;
; 1) the following things must be defined:
;	$FLTEND	EQU	$  ;place just before END st.
;	labels must define:
;		- start of actual filter
;		- length of actual filter
; 2) a call of the macro RELO must be done in the program
;	at the point where relocation is to take place.
; 4) a call of the macro MOVE is done after setup,
;	whenever the filter is to be moved to high memory.
; 4) the manipulation of HIGH$ is done in RELO,
;	and is Mod I/III independent. So, DON'T
;	MANIPULATE HIGH$.
; 5) linkage to the actual DCB is not performed, so
;	you must set up the linkage.
; 5) after each reference to a memory address that
	will changer after relocation, enter a
;	call to the MARK macro.  This automatically sets
;	up an entry in the relocation table, so you
;	don't have to.
;
$PASS	DEFL	$PASS+1		;EDAS pass #
$RELTBL	DEFL	$FLTEND		;build relo. tbl at end
$RELPTR	DEFL	$RELTBL		;init. pseudo pc
  IFEQ	$PASS,2			;if beginning of 2nd pass
$RELNUM	DEFL	$RELCTR		;record # of rel. words
  ENDIF
;
MARK	MACRO
$PCTEMP	DEFL	$		;save PC
	ORG	$RELPTR		;save data word in reltbl
	DW	$PCTEMP-2	;and point to relo data
$RELCTR	DEFL	$RELCTR+1	;relo. word count
$RELPTR	DEFL	$
	ORG	$PCTEMP		;all is restored
	ENDM
;
;   Macro to adjust addresses in the filter
;
RELO	MACRO	#START,#LEN
	LD	A,(125H)	;get Mod I/III flag
	CP	'I'		;is it mod. 3?
	LD	IX,4049H	;default to no.
	JR	NZ,$+6		;not mod 3, skip
	LD	IX,4411H	;set up for mod 3
	LD	L,(IX)		;get LSByte of high$
	LD	H,(IX+1)	;and MSByte
	LD	BC,#LEN		;LENGTH OF FILTER
	XOR	A		;CLEAR CARRY
	SBC	HL,BC		;CALCULATE STARTING ADDR
	LD	(IX),L		;store new value
	LD	(IX+1),H	;into high$
	INC	HL		;point to actual start
;***
;*	COMPUTE OFFSET FROM ORIGINAL ADDRESS OF FILTER
;***
	LD	DE,#START
	XOR	A		;CLEAR CARRY
	SBC	HL,DE		;COMPUTE OFFSET
	EX	DE,HL		;save offset in DE
	LD	BC,$RELNUM	;# of relocated words
	LD	IX,$RELTBL	;base of relo. table
$RELO$1
	LD	A,B
	OR	C		;is the count = 0?
	JR	Z,$RELO$2	;yes, exit loop
	LD	L,(IX+0)	;get LSB of address
	LD	H,(IX+1)	;and MSB
	PUSH	HL
	POP	IY		;point to word to relo.
	LD	L,(IY)		;get address
	LD	H,(IY+1)
	ADD	HL,DE		;add offset
	LD	(IY),L		;and store new value
	LD	(IY+1),H	; into the filter
	INC	IX
	INC	IX		;point to next relo. word
	DEC	BC
	JR	$RELO$1		;and loop
$RELO$2
	ENDM
;***
;*	MOVE FILTER TO HIGH MEMORY
;***
MOVE	MACRO	#START,#LEN
	LD	HL,#START	;source of move
	LD	DE,#START	;relocated start!
	MARK
	LD	BC,#LEN		;length of move
	LDIR			;MOVE IT
	ENDM			;end of RELO!
;
;*** end of autorel ***
/exit
